@@ -1,22 +1,23 @@
 /* tnt/httprequest.h
  * Copyright (C) 2003-2005 Tommi Maekitalo
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
- * is provided AS IS, WITHOUT ANY WARRANTY; without even the implied
- * warranty of MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and
- * NON-INFRINGEMENT.  See the GNU General Public License for more details.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
+
 #ifndef TNT_HTTPREQUEST_H
 #define TNT_HTTPREQUEST_H
 
@@ -81,17 +82,7 @@ namespace tnt
       mutable std::string serverAddrStr;
 
     public:
-      HttpRequest()
-        : ssl(false),
-          locale_init(false),
-          encodingRead(false),
-          requestScope(0),
-          applicationScope(0),
-          threadScope(0),
-          sessionScope(0),
-          applicationScopeLocked(false),
-          sessionScopeLocked(false)
-        { }
+      HttpRequest();
       HttpRequest(const std::string& url);
       HttpRequest(const HttpRequest& r);
       ~HttpRequest();
@@ -108,7 +99,8 @@ namespace tnt
       args_type& getArgs()                         { return args; }
 
       args_type::const_reference getArgDef(args_type::size_type n,
-        const std::string& def = std::string()) const;
+        const std::string& def = std::string()) const
+        { return args.size() > n ? args[n] : def; }
       args_type::const_reference getArg(args_type::size_type n) const
                                                    { return args[n]; }
       args_type::size_type getArgsCount() const    { return args.size(); }
@@ -158,6 +150,8 @@ namespace tnt
       const Encoding& getEncoding() const;
       std::string getUserAgent() const
         { return getHeader(httpheader::userAgent); }
+      std::string getHost() const
+        { return getHeader(httpheader::host); }
 
       bool keepAlive() const;
 
