@@ -1,22 +1,23 @@
 /* parser.cpp
  * Copyright (C) 2003-2005 Tommi Maekitalo
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
- * is provided AS IS, WITHOUT ANY WARRANTY; without even the implied
- * warranty of MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and
- * NON-INFRINGEMENT.  See the GNU General Public License for more details.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
+
 #include "tnt/ecpp/parser.h"
 #include "tnt/ecpp/parsehandler.h"
 #include <sstream>
@@ -35,7 +36,19 @@ namespace tnt
 
     void Parser::doInclude(const std::string& file)
     {
+      log_debug("include \"" << file << '"');
+
+      std::string fullname = file;
       std::ifstream inp(file.c_str());
+
+      for (includes_type::const_iterator it = includes.begin();
+           !inp && it != includes.end(); ++it)
+      {
+        fullname = *it + '/' + file;
+        log_debug("try include \"" << fullname << '"');
+        inp.open(fullname.c_str());
+      }
+
       if (!inp)
       {
         std::ostringstream msg;
@@ -46,11 +59,11 @@ namespace tnt
       {
         std::string curfileSave = curfile;
         unsigned curlineSave = curline;
-        curfile = file;
+        curfile = fullname;
         curline = 0;
 
-        log_debug("onInclude(\"" << file << "\")");
-        handler.onInclude(file);
+        log_debug("onInclude(\"" << fullname << "\")");
+        handler.onInclude(fullname);
 
         parsePriv(inp);
 
@@ -63,8 +76,8 @@ namespace tnt
         throw;
       }
 
-      log_debug("onIncludeEnd(\"" << file << "\")");
-      handler.onIncludeEnd(file);
+      log_debug("onIncludeEnd(\"" << fullname << "\")");
+      handler.onIncludeEnd(fullname);
     }
 
     void Parser::parsePriv(std::istream& in)
@@ -91,66 +104,70 @@ namespace tnt
         state_cppe1,
         state_cppetag,
         state_cppstring,  // 20
+        state_cppstringesc,
+        state_cppchar,
+        state_cppcharesc,
+        state_cppchare,
         state_nl,
         state_cpp1,
         state_args0,
         state_args0comment,
         state_argsvar,
-        state_argsvare,
+        state_argsvare,  // 30
         state_argsval,
         state_argsvalstring,
         state_argscomment0,
-        state_argscomment,  // 30
+        state_argscomment,
         state_argsvalcomment0,
         state_attr0,
         state_attr0comment,
         state_attrvar,
         state_attrvare,
-        state_attrval,
+        state_attrval,  // 40
         state_attrvalstring,
         state_attrcomment0,
         state_attrcomment,
-        state_attrvalcomment0,  // 40
+        state_attrvalcomment0,
         state_call0,
         state_callname_expr,
         state_callname_string,
         state_callname,
         state_call_cpparg0,
-        state_call_cpparg,
+        state_call_cpparg,   // 50
         state_callend,
         state_callarg0,
         state_callarg,
-        state_callarge,   // 50
+        state_callarge,
         state_callval_expr,
         state_callval_string,
         state_callval_word,
         state_callval0,
         state_callvale,
-        state_comment,
+        state_comment,  // 60
         state_commente,
         state_compe0,
         state_compe,
-        state_cond,  // 60
+        state_cond,
         state_condexpr,
         state_condexpre,
         state_include0,
         state_include1,
         state_scopearg0,
-        state_scopearg,
+        state_scopearg,  // 70
         state_scopeargeq,
         state_scopeargval0,
         state_scopeargval,
-        state_scopevale,  // 70
+        state_scopevale,
         state_scope0,
         state_scope,
         state_scopeinit,
         state_scopee,
         state_scopee0,
-        state_scopecomment0,
+        state_scopecomment0,  // 80
         state_scopecomment,
         state_endcall0,
         state_endcall,
-        state_endcalle  // 80
+        state_endcalle
       };
 
       state_type state = state_nl;
@@ -571,6 +588,8 @@ namespace tnt
               code += ch;
               if (ch == '"')
                 state = state_cppstring;
+              else if (ch == '\'')
+                state = state_cppchar;
             }
             break;
 
@@ -578,6 +597,32 @@ namespace tnt
             code += ch;
             if (ch == '"')
               state = state_cpp;
+            else if (ch == '\\')
+              state = state_cppstringesc;
+            break;
+
+          case state_cppstringesc:
+            code += ch;
+            state = state_cppstring;
+            break;
+
+          case state_cppchar:
+            code += ch;
+            if (ch == '\\')
+              state = state_cppcharesc;
+            else
+              state = state_cppchare;
+            break;
+
+          case state_cppcharesc:
+            code += ch;
+            state = state_cppchare;
+            break;
+
+          case state_cppchare:
+            code += ch;
+            if (ch == '\'')
+              state = state_cpp;
             break;
 
           case state_cppse:
@@ -615,7 +660,8 @@ namespace tnt
               state = state_cppetag;
             else
             {
-              code += "</%";
+              code += "</";
+              code += ch;
               state = state_cpp;
             }
             break;
@@ -632,11 +678,6 @@ namespace tnt
                 log_debug("onPre(\"" << code << "\")");
                 handler.onPre(code);
               }
-              else if (tag == "declare")
-              {
-                log_debug("onDeclare(\"" << code << "\")");
-                handler.onDeclare(code);
-              }
               else if (tag == "init")
               {
                 log_debug("onInit(\"" << code << "\")");
@@ -647,11 +688,6 @@ namespace tnt
                 log_debug("onCleanup(\"" << code << "\")");
                 handler.onCleanup(code);
               }
-              else if (tag == "declare_shared")
-              {
-                log_debug("onDeclareShared(\"" << code << "\")");
-                handler.onDeclareShared(code);
-              }
               else if (tag == "shared")
               {
                 log_debug("onShared(\"" << code << "\")");
