@@ -2,21 +2,22 @@
  * Copyright (C) 2005 Tommi Maekitalo
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
- * is provided AS IS, WITHOUT ANY WARRANTY; without even the implied
- * warranty of MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and
- * NON-INFRINGEMENT.  See the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
+
 #ifndef TNT_ECPPC_BODY_H
 #define TNT_ECPPC_BODY_H
 
@@ -39,6 +40,7 @@ namespace tnt
 
         unsigned curline;
         std::string curfile;
+        static bool linenumbersEnabled;
 
       public:
         Bodypart()
@@ -57,6 +59,9 @@ namespace tnt
 
         virtual void getBody(std::ostream& out) const = 0;
         void printLine(std::ostream& out) const;
+
+        static void enableLinenumbers(bool sw = true)  { linenumbersEnabled = sw; }
+        static bool isLinenumbersEnabled()             { return linenumbersEnabled; }
     };
 
     class BodypartStatic : public Bodypart
